@@ -25,6 +25,7 @@ __license__ = "GPL"
 
 import os
 import sys
+import unittest
 
 from ..client import fail2banregex
 from ..client.fail2banregex import Fail2banRegex, get_opt_parser, exec_command_line, output, str2LogLevel
@@ -51,6 +52,10 @@ def _Fail2banRegex(*args):
 		logSys.setLevel(str2LogLevel(opts.log_level))
 	return (opts, args, Fail2banRegex(opts))
 
+def _test_exec(*args):
+	(opts, args, fail2banRegex) = _Fail2banRegex(*args)
+	return fail2banRegex.start(args)
+
 class ExitException(Exception):
 	def __init__(self, code):
 		self.code = code
@@ -75,23 +80,27 @@ def _test_exec_command_line(*args):
 		sys.stderr = _org['stderr']
 	return _exit_code
 
+STR_00 = "Dec 31 11:59:59 [sshd] error: PAM: Authentication failure for kevin from 192.0.2.0"
+
+RE_00 = r"(?:(?:Authentication failure|Failed [-/\w+]+) for(?: [iI](?:llegal|nvalid) user)?|[Ii](?:llegal|nvalid) user|ROOT LOGIN REFUSED) .*(?: from|FROM) <HOST>"
+RE_00_ID = r"Authentication failure for <F-ID>.*?</F-ID> from <HOST>$"
+RE_00_USER = r"Authentication failure for <F-USER>.*?</F-USER> from <HOST>$"
+
+FILENAME_01 = os.path.join(TEST_FILES_DIR, "testcase01.log")
+FILENAME_02 = os.path.join(TEST_FILES_DIR, "testcase02.log")
+FILENAME_WRONGCHAR = os.path.join(TEST_FILES_DIR, "testcase-wrong-char.log")
+
+FILENAME_SSHD = os.path.join(TEST_FILES_DIR, "logs", "sshd")
+FILTER_SSHD = os.path.join(CONFIG_DIR, 'filter.d', 'sshd.conf')
+FILENAME_ZZZ_SSHD = os.path.join(TEST_FILES_DIR, 'zzz-sshd-obsolete-multiline.log')
+FILTER_ZZZ_SSHD = os.path.join(TEST_CONFIG_DIR, 'filter.d', 'zzz-sshd-obsolete-multiline.conf')
+
+FILENAME_ZZZ_GEN = os.path.join(TEST_FILES_DIR, "logs", "zzz-generic-example")
+FILTER_ZZZ_GEN = os.path.join(TEST_CONFIG_DIR, 'filter.d', 'zzz-generic-example.conf')
+
 
 class Fail2banRegexTest(LogCaptureTestCase):
 
-	RE_00 = r"(?:(?:Authentication failure|Failed [-/\w+]+) for(?: [iI](?:llegal|nvalid) user)?|[Ii](?:llegal|nvalid) user|ROOT LOGIN REFUSED) .*(?: from|FROM) <HOST>"
-
-	FILENAME_01 = os.path.join(TEST_FILES_DIR, "testcase01.log")
-	FILENAME_02 = os.path.join(TEST_FILES_DIR, "testcase02.log")
-	FILENAME_WRONGCHAR = os.path.join(TEST_FILES_DIR, "testcase-wrong-char.log")
-
-	FILENAME_SSHD = os.path.join(TEST_FILES_DIR, "logs", "sshd")
-	FILTER_SSHD = os.path.join(CONFIG_DIR, 'filter.d', 'sshd.conf')
-	FILENAME_ZZZ_SSHD = os.path.join(TEST_FILES_DIR, 'zzz-sshd-obsolete-multiline.log')
-	FILTER_ZZZ_SSHD = os.path.join(TEST_CONFIG_DIR, 'filter.d', 'zzz-sshd-obsolete-multiline.conf')
-
-	FILENAME_ZZZ_GEN = os.path.join(TEST_FILES_DIR, "logs", "zzz-generic-example")
-	FILTER_ZZZ_GEN = os.path.join(TEST_CONFIG_DIR, 'filter.d', 'zzz-generic-example.conf')
-
 	def setUp(self):
 		"""Call before every test case."""
 		LogCaptureTestCase.setUp(self)
@@ -103,57 +112,50 @@ class Fail2banRegexTest(LogCaptureTestCase):
 		tearDownMyTime()
 
 	def testWrongRE(self):
-		(opts, args, fail2banRegex) = _Fail2banRegex(
+		self.assertFalse(_test_exec(
 			"test", r".** from <HOST>$"
-		)
-		self.assertFalse(fail2banRegex.start(args))
+		))
 		self.assertLogged("Unable to compile regular expression")
 
 	def testWrongIngnoreRE(self):
-		(opts, args, fail2banRegex) = _Fail2banRegex(
+		self.assertFalse(_test_exec(
 			"--datepattern", "{^LN-BEG}EPOCH",
 			"test", r".*? from <HOST>$", r".**"
-		)
-		self.assertFalse(fail2banRegex.start(args))
+		))
 		self.assertLogged("Unable to compile regular expression")
 
 	def testDirectFound(self):
-		(opts, args, fail2banRegex) = _Fail2banRegex(
-			"--datepattern", "^(?:%a )?%b %d %H:%M:%S(?:\.%f)?(?: %ExY)?",
+		self.assertTrue(_test_exec(
+			"--datepattern", r"^(?:%a )?%b %d %H:%M:%S(?:\.%f)?(?: %ExY)?",
 			"--print-all-matched", "--print-no-missed",
-			"Dec 31 11:59:59 [sshd] error: PAM: Authentication failure for kevin from 192.0.2.0",
+			STR_00,
 			r"Authentication failure for .*? from <HOST>$"
-		)
-		self.assertTrue(fail2banRegex.start(args))
+		))
 		self.assertLogged('Lines: 1 lines, 0 ignored, 1 matched, 0 missed')
 
 	def testDirectNotFound(self):
-		(opts, args, fail2banRegex) = _Fail2banRegex(
+		self.assertTrue(_test_exec(
 			"--print-all-missed",
-			"Dec 31 11:59:59 [sshd] error: PAM: Authentication failure for kevin from 192.0.2.0",
+			STR_00,
 			r"XYZ from <HOST>$"
-		)
-		self.assertTrue(fail2banRegex.start(args))
+		))
 		self.assertLogged('Lines: 1 lines, 0 ignored, 0 matched, 1 missed')
 
 	def testDirectIgnored(self):
-		(opts, args, fail2banRegex) = _Fail2banRegex(
+		self.assertTrue(_test_exec(
 			"--print-all-ignored",
-			"Dec 31 11:59:59 [sshd] error: PAM: Authentication failure for kevin from 192.0.2.0",
+			STR_00,
 			r"Authentication failure for .*? from <HOST>$",
 			r"kevin from 192.0.2.0$"
-		)
-		self.assertTrue(fail2banRegex.start(args))
+		))
 		self.assertLogged('Lines: 1 lines, 1 ignored, 0 matched, 0 missed')
 
 	def testDirectRE_1(self):
-		(opts, args, fail2banRegex) = _Fail2banRegex(
-			"--datepattern", "^(?:%a )?%b %d %H:%M:%S(?:\.%f)?(?: %ExY)?",
+		self.assertTrue(_test_exec(
+			"--datepattern", r"^(?:%a )?%b %d %H:%M:%S(?:\.%f)?(?: %ExY)?",
 			"--print-all-matched",
-			Fail2banRegexTest.FILENAME_01, 
-			Fail2banRegexTest.RE_00
-		)
-		self.assertTrue(fail2banRegex.start(args))
+			FILENAME_01, RE_00
+		))
 		self.assertLogged('Lines: 19 lines, 0 ignored, 13 matched, 6 missed')
 
 		self.assertLogged('Error decoding line');
@@ -163,69 +165,78 @@ class Fail2banRegexTest(LogCaptureTestCase):
 		self.assertLogged('Dec 31 11:59:59 [sshd] error: PAM: Authentication failure for kevin from 87.142.124.10')
 
 	def testDirectRE_1raw(self):
-		(opts, args, fail2banRegex) = _Fail2banRegex(
-			"--datepattern", "^(?:%a )?%b %d %H:%M:%S(?:\.%f)?(?: %ExY)?",
+		self.assertTrue(_test_exec(
+			"--datepattern", r"^(?:%a )?%b %d %H:%M:%S(?:\.%f)?(?: %ExY)?",
 			"--print-all-matched", "--raw",
-			Fail2banRegexTest.FILENAME_01, 
-			Fail2banRegexTest.RE_00
-		)
-		self.assertTrue(fail2banRegex.start(args))
+			FILENAME_01, RE_00
+		))
 		self.assertLogged('Lines: 19 lines, 0 ignored, 16 matched, 3 missed')
 
 	def testDirectRE_1raw_noDns(self):
-		(opts, args, fail2banRegex) = _Fail2banRegex(
-			"--datepattern", "^(?:%a )?%b %d %H:%M:%S(?:\.%f)?(?: %ExY)?",
+		self.assertTrue(_test_exec(
+			"--datepattern", r"^(?:%a )?%b %d %H:%M:%S(?:\.%f)?(?: %ExY)?",
 			"--print-all-matched", "--raw", "--usedns=no",
-			Fail2banRegexTest.FILENAME_01, 
-			Fail2banRegexTest.RE_00
-		)
-		self.assertTrue(fail2banRegex.start(args))
+			FILENAME_01, RE_00
+		))
 		self.assertLogged('Lines: 19 lines, 0 ignored, 13 matched, 6 missed')
+		# usage of <F-ID>\S+</F-ID> causes raw handling automatically:
+		self.pruneLog()
+		self.assertTrue(_test_exec(
+			"-d", "^Epoch",
+			"1490349000 test failed.dns.ch", "^\s*test <F-ID>\S+</F-ID>"
+		))
+		self.assertLogged('Lines: 1 lines, 0 ignored, 1 matched, 0 missed', all=True)
+		self.assertNotLogged('Unable to find a corresponding IP address')
 
 	def testDirectRE_2(self):
-		(opts, args, fail2banRegex) = _Fail2banRegex(
-			"--datepattern", "^(?:%a )?%b %d %H:%M:%S(?:\.%f)?(?: %ExY)?",
+		self.assertTrue(_test_exec(
+			"--datepattern", r"^(?:%a )?%b %d %H:%M:%S(?:\.%f)?(?: %ExY)?",
 			"--print-all-matched",
-			Fail2banRegexTest.FILENAME_02, 
-			Fail2banRegexTest.RE_00
-		)
-		self.assertTrue(fail2banRegex.start(args))
+			FILENAME_02, RE_00
+		))
 		self.assertLogged('Lines: 13 lines, 0 ignored, 5 matched, 8 missed')
 
 	def testVerbose(self):
-		(opts, args, fail2banRegex) = _Fail2banRegex(
-			"--datepattern", "^(?:%a )?%b %d %H:%M:%S(?:\.%f)?(?: %ExY)?",
+		self.assertTrue(_test_exec(
+			"--datepattern", r"^(?:%a )?%b %d %H:%M:%S(?:\.%f)?(?: %ExY)?",
 			"--timezone", "UTC+0200",
 			"--verbose", "--verbose-date", "--print-no-missed",
-			Fail2banRegexTest.FILENAME_02, 
-			Fail2banRegexTest.RE_00
-		)
-		self.assertTrue(fail2banRegex.start(args))
+			FILENAME_02, RE_00
+		))
 		self.assertLogged('Lines: 13 lines, 0 ignored, 5 matched, 8 missed')
 
 		self.assertLogged('141.3.81.106  Sun Aug 14 11:53:59 2005')
 		self.assertLogged('141.3.81.106  Sun Aug 14 11:54:59 2005')
 
 	def testVerboseFullSshd(self):
-		(opts, args, fail2banRegex) = _Fail2banRegex(
-			"-l", "notice", # put down log-level, because of too many debug-messages
+		self.assertTrue(_test_exec(
+		"-l", "notice", # put down log-level, because of too many debug-messages
 			"-v", "--verbose-date", "--print-all-matched", "--print-all-ignored",
 			"-c", CONFIG_DIR,
-			Fail2banRegexTest.FILENAME_SSHD, "sshd"
-		)
-		self.assertTrue(fail2banRegex.start(args))
+			FILENAME_SSHD, "sshd"
+		))
 		# test failure line and not-failure lines both presents:
 		self.assertLogged("[29116]: User root not allowed because account is locked",
 			"[29116]: Received disconnect from 1.2.3.4", all=True)
+		self.pruneLog()
+		# show real options:
+		self.assertTrue(_test_exec(
+		"-l", "notice", # put down log-level, because of too many debug-messages
+			"-vv", "-c", CONFIG_DIR,
+			"Dec 31 11:59:59 [sshd] error: PAM: Authentication failure for kevin from 192.0.2.1",
+			"sshd[logtype=short]"
+		))
+		# tet logtype is specified and set in real options:
+		self.assertLogged("Real  filter options :", "'logtype': 'short'", all=True)
+		self.assertNotLogged("'logtype': 'file'", "'logtype': 'journal'", all=True)
 
 	def testFastSshd(self):
-		(opts, args, fail2banRegex) = _Fail2banRegex(
-			"-l", "notice", # put down log-level, because of too many debug-messages
+		self.assertTrue(_test_exec(
+		"-l", "notice", # put down log-level, because of too many debug-messages
 			"--print-all-matched",
 			"-c", CONFIG_DIR,
-			Fail2banRegexTest.FILENAME_ZZZ_SSHD, "sshd.conf[mode=normal]"
-		)
-		self.assertTrue(fail2banRegex.start(args))
+			FILENAME_ZZZ_SSHD, "sshd.conf[mode=normal]"
+		))
 		# test failure line and all not-failure lines presents:
 		self.assertLogged(
 			"[29116]: Connection from 192.0.2.4",
@@ -234,80 +245,107 @@ class Fail2banRegexTest(LogCaptureTestCase):
 
 	def testMultilineSshd(self):
 		# by the way test of missing lines by multiline in `for bufLine in orgLineBuffer[int(fullBuffer):]`
-		(opts, args, fail2banRegex) = _Fail2banRegex(
-			"-l", "notice", # put down log-level, because of too many debug-messages
+		self.assertTrue(_test_exec(
+		"-l", "notice", # put down log-level, because of too many debug-messages
 			"--print-all-matched", "--print-all-missed",
-			"-c", os.path.dirname(Fail2banRegexTest.FILTER_ZZZ_SSHD),
-			Fail2banRegexTest.FILENAME_ZZZ_SSHD, os.path.basename(Fail2banRegexTest.FILTER_ZZZ_SSHD)
-		)
-		self.assertTrue(fail2banRegex.start(args))
+			"-c", os.path.dirname(FILTER_ZZZ_SSHD),
+			FILENAME_ZZZ_SSHD, os.path.basename(FILTER_ZZZ_SSHD)
+		))
 		# test "failure" line presents (2nd part only, because multiline fewer precise):
 		self.assertLogged(
 			"[29116]: Received disconnect from 192.0.2.4", all=True)
 
 	def testFullGeneric(self):
 		# by the way test of ignoreregex (specified in filter file)...
-		(opts, args, fail2banRegex) = _Fail2banRegex(
-			"-l", "notice", # put down log-level, because of too many debug-messages
-			Fail2banRegexTest.FILENAME_ZZZ_GEN, Fail2banRegexTest.FILTER_ZZZ_GEN+"[mode=test]"
-		)
-		self.assertTrue(fail2banRegex.start(args))
+		self.assertTrue(_test_exec(
+		"-l", "notice", # put down log-level, because of too many debug-messages
+			FILENAME_ZZZ_GEN, FILTER_ZZZ_GEN+"[mode=test]"
+		))
 
 	def testDirectMultilineBuf(self):
 		# test it with some pre-lines also to cover correct buffer scrolling (all multi-lines printed):
 		for preLines in (0, 20):
 			self.pruneLog("[test-phase %s]" % preLines)
-			(opts, args, fail2banRegex) = _Fail2banRegex(
+			self.assertTrue(_test_exec(
 				"--usedns", "no", "-d", "^Epoch", "--print-all-matched", "--maxlines", "5", 
 				("1490349000 TEST-NL\n"*preLines) + 
 				"1490349000 FAIL\n1490349000 TEST1\n1490349001 TEST2\n1490349001 HOST 192.0.2.34",
 				r"^\s*FAIL\s*$<SKIPLINES>^\s*HOST <HOST>\s*$"
-			)
-			self.assertTrue(fail2banRegex.start(args))
+			))
 			self.assertLogged('Lines: %s lines, 0 ignored, 2 matched, %s missed' % (preLines+4, preLines+2))
 			# both matched lines were printed:
 			self.assertLogged("|  1490349000 FAIL", "|  1490349001 HOST 192.0.2.34", all=True)
 
 
 	def testDirectMultilineBufDebuggex(self):
-		(opts, args, fail2banRegex) = _Fail2banRegex(
+		self.assertTrue(_test_exec(
 			"--usedns", "no", "-d", "^Epoch", "--debuggex", "--print-all-matched", "--maxlines", "5",
 			"1490349000 FAIL\n1490349000 TEST1\n1490349001 TEST2\n1490349001 HOST 192.0.2.34",
 			r"^\s*FAIL\s*$<SKIPLINES>^\s*HOST <HOST>\s*$"
-		)
-		self.assertTrue(fail2banRegex.start(args))
+		))
 		self.assertLogged('Lines: 4 lines, 0 ignored, 2 matched, 2 missed')
 		# the sequence in args-dict is currently undefined (so can be 1st argument)
 		self.assertLogged("&flags=m", "?flags=m")
 
 	def testSinglelineWithNLinContent(self):
 		# 
-		(opts, args, fail2banRegex) = _Fail2banRegex(
+		self.assertTrue(_test_exec(
 			"--usedns", "no", "-d", "^Epoch", "--print-all-matched",
 			"1490349000 FAIL: failure\nhost: 192.0.2.35",
 			r"^\s*FAIL:\s*.*\nhost:\s+<HOST>$"
-		)
-		self.assertTrue(fail2banRegex.start(args))
+		))
 		self.assertLogged('Lines: 1 lines, 0 ignored, 1 matched, 0 missed')
 
 	def testRegexEpochPatterns(self):
-		(opts, args, fail2banRegex) = _Fail2banRegex(
+		self.assertTrue(_test_exec(
 			"-r", "-d", r"^\[{LEPOCH}\]\s+", "--maxlines", "5",
 			"[1516469849] 192.0.2.1 FAIL: failure\n"
 			"[1516469849551] 192.0.2.2 FAIL: failure\n"
 			"[1516469849551000] 192.0.2.3 FAIL: failure\n"
 			"[1516469849551.000] 192.0.2.4 FAIL: failure",
 			r"^<HOST> FAIL\b"
-		)
-		self.assertTrue(fail2banRegex.start(args))
+		))
 		self.assertLogged('Lines: 4 lines, 0 ignored, 4 matched, 0 missed')
 
+	def testRegexSubnet(self):
+		self.assertTrue(_test_exec(
+			"-vv", "-d", r"^\[{LEPOCH}\]\s+", "--maxlines", "5",
+			"[1516469849] 192.0.2.1 FAIL: failure\n"
+			"[1516469849] 192.0.2.1/24 FAIL: failure\n"
+			"[1516469849] 2001:DB8:FF:FF::1 FAIL: failure\n"
+			"[1516469849] 2001:DB8:FF:FF::1/60 FAIL: failure\n",
+			r"^<SUBNET> FAIL\b"
+		))
+		self.assertLogged('Lines: 4 lines, 0 ignored, 4 matched, 0 missed')
+		self.assertLogged('192.0.2.0/24', '2001:db8:ff:f0::/60', all=True)
+
+	def testFrmtOutput(self):
+		# id/ip only:
+		self.assertTrue(_test_exec('-o', 'id', STR_00, RE_00_ID))
+		self.assertLogged('kevin')
+		self.pruneLog()
+		# row with id :
+		self.assertTrue(_test_exec('-o', 'row', STR_00, RE_00_ID))
+		self.assertLogged("['kevin'", "'ip4': '192.0.2.0'", "'fid': 'kevin'", all=True)
+		self.pruneLog()
+		# row with ip :
+		self.assertTrue(_test_exec('-o', 'row', STR_00, RE_00_USER))
+		self.assertLogged("['192.0.2.0'", "'ip4': '192.0.2.0'", "'user': 'kevin'", all=True)
+		self.pruneLog()
+		# log msg :
+		self.assertTrue(_test_exec('-o', 'msg', STR_00, RE_00_USER))
+		self.assertLogged(STR_00)
+		self.pruneLog()
+		# item of match (user):
+		self.assertTrue(_test_exec('-o', 'user', STR_00, RE_00_USER))
+		self.assertLogged('kevin')
+		self.pruneLog()
+
 	def testWrongFilterFile(self):
 		# use test log as filter file to cover eror cases...
-		(opts, args, fail2banRegex) = _Fail2banRegex(
-			Fail2banRegexTest.FILENAME_ZZZ_GEN, Fail2banRegexTest.FILENAME_ZZZ_GEN
-		)
-		self.assertFalse(fail2banRegex.start(args))
+		self.assertFalse(_test_exec(
+			FILENAME_ZZZ_GEN, FILENAME_ZZZ_GEN
+		))
 
 	def _reset(self):
 		# reset global warn-counter:
@@ -315,13 +353,13 @@ class Fail2banRegexTest(LogCaptureTestCase):
 		_decode_line_warn.clear()
 
 	def testWronChar(self):
+		unittest.F2B.SkipIfCfgMissing(stock=True)
 		self._reset()
-		(opts, args, fail2banRegex) = _Fail2banRegex(
-			"-l", "notice", # put down log-level, because of too many debug-messages
-			"--datepattern", "^(?:%a )?%b %d %H:%M:%S(?:\.%f)?(?: %ExY)?",
-			Fail2banRegexTest.FILENAME_WRONGCHAR, Fail2banRegexTest.FILTER_SSHD
-		)
-		self.assertTrue(fail2banRegex.start(args))
+		self.assertTrue(_test_exec(
+		"-l", "notice", # put down log-level, because of too many debug-messages
+			"--datepattern", r"^(?:%a )?%b %d %H:%M:%S(?:\.%f)?(?: %ExY)?",
+			FILENAME_WRONGCHAR, FILTER_SSHD
+		))
 		self.assertLogged('Lines: 4 lines, 0 ignored, 2 matched, 2 missed')
 
 		self.assertLogged('Error decoding line')
@@ -331,15 +369,15 @@ class Fail2banRegexTest(LogCaptureTestCase):
 		self.assertLogged('Nov  8 00:16:12 main sshd[32547]: pam_succeed_if(sshd:auth): error retrieving information about user llinco')
 
 	def testWronCharDebuggex(self):
+		unittest.F2B.SkipIfCfgMissing(stock=True)
 		self._reset()
-		(opts, args, fail2banRegex) = _Fail2banRegex(
-			"-l", "notice", # put down log-level, because of too many debug-messages
-			"--datepattern", "^(?:%a )?%b %d %H:%M:%S(?:\.%f)?(?: %ExY)?",
+		self.assertTrue(_test_exec(
+		"-l", "notice", # put down log-level, because of too many debug-messages
+			"--datepattern", r"^(?:%a )?%b %d %H:%M:%S(?:\.%f)?(?: %ExY)?",
 			"--debuggex", "--print-all-matched",
-			Fail2banRegexTest.FILENAME_WRONGCHAR, Fail2banRegexTest.FILTER_SSHD,
+			FILENAME_WRONGCHAR, FILTER_SSHD,
 			r"llinco[^\\]"
-		)
-		self.assertTrue(fail2banRegex.start(args))
+		))
 		self.assertLogged('Error decoding line')
 		self.assertLogged('Lines: 4 lines, 1 ignored, 2 matched, 1 missed')
 
@@ -356,15 +394,48 @@ class Fail2banRegexTest(LogCaptureTestCase):
 	def testExecCmdLine_Direct(self):
 		self.assertEqual(_test_exec_command_line(
 			'-l', 'info',
-			"Dec 31 11:59:59 [sshd] error: PAM: Authentication failure for kevin from 192.0.2.0",
-			r"Authentication failure for .*? from <HOST>$"
+			STR_00, r"Authentication failure for .*? from <HOST>$"
 		), 0)
 		self.assertLogged('Lines: 1 lines, 0 ignored, 1 matched, 0 missed')
 		
 	def testExecCmdLine_MissFailID(self):
 		self.assertNotEqual(_test_exec_command_line(
 			'-l', 'info',
-			"Dec 31 11:59:59 [sshd] error: PAM: Authentication failure for kevin from 192.0.2.0",
-			r"Authentication failure"
+			STR_00, r"Authentication failure"
 		), 0)
 		self.assertLogged('No failure-id group in ')
+
+	def testExecCmdLine_ErrorParam(self):
+		# single line error:
+		self.assertNotEqual(_test_exec_command_line(
+			'-l', 'notice', '-d', '%:%.%-', 'LOG', 'RE'
+		), 0)
+		self.assertLogged('ERROR: Failed to set datepattern')
+		# verbose (traceback/callstack):
+		self.pruneLog()
+		self.assertNotEqual(_test_exec_command_line(
+			'-v', '-d', '%:%.%-', 'LOG', 'RE'
+		), 0)
+		self.assertLogged('Failed to set datepattern')
+
+	def testLogtypeSystemdJournal(self): # pragma: no cover
+		if not fail2banregex.FilterSystemd:
+			raise unittest.SkipTest('Skip test because no systemd backand available')
+		self.assertTrue(_test_exec(
+			"systemd-journal", FILTER_ZZZ_GEN
+			  +'[journalmatch="SYSLOG_IDENTIFIER=\x01\x02dummy\x02\x01",'
+				+' failregex="^\x00\x01\x02dummy regex, never match <F-ID>xxx</F-ID>"]'
+		))
+		self.assertLogged("'logtype': 'journal'")
+		self.assertNotLogged("'logtype': 'file'")
+		self.assertLogged('Lines: 0 lines, 0 ignored, 0 matched, 0 missed')
+		self.pruneLog()
+		# logtype specified explicitly (should win in filter):
+		self.assertTrue(_test_exec(
+			"systemd-journal", FILTER_ZZZ_GEN
+			  +'[logtype=file,'
+			  +' journalmatch="SYSLOG_IDENTIFIER=\x01\x02dummy\x02\x01",'
+				+' failregex="^\x00\x01\x02dummy regex, never match <F-ID>xxx</F-ID>"]'
+		))
+		self.assertLogged("'logtype': 'file'")
+		self.assertNotLogged("'logtype': 'journal'")
