@@ -138,7 +138,7 @@ class FilterSamplesRegex(unittest.TestCase):
 
 	@staticmethod
 	def _filterOptions(opts):
-				return dict((k, v) for k, v in opts.items() if not k.startswith('test.'))
+				return dict((k, v) for k, v in list(opts.items()) if not k.startswith('test.'))
 		
 def testSampleRegexsFactory(name, basedir):
 	def testFilter(self):
@@ -254,7 +254,7 @@ def testSampleRegexsFactory(name, basedir):
 							"Multiple regexs matched %r" % ([x[0] for x in ret]))
 
 						# Verify match captures (at least fid/host) and timestamp as expected
-						for k, v in faildata.items():
+						for k, v in list(faildata.items()):
 							if k not in ("time", "match", "desc", "filter"):
 								fv = fail.get(k, None)
 								if fv is None:
@@ -296,7 +296,7 @@ def testSampleRegexsFactory(name, basedir):
 								'\n'.join(pprint.pformat(fail).splitlines())))
 
 		# check missing samples for regex using each filter-options combination:
-		for fltName, flt in self._filters.items():
+		for fltName, flt in list(self._filters.items()):
 			flt, regexsUsedIdx = flt
 			regexList = flt.getFailRegex()
 			for failRegexIndex, failRegex in enumerate(regexList):
